--[==[
    Aleri

    All Rights Reserved 
    
    © stpain 2020
]==]--

local AleriName, Aleri = ...
local L = Aleri.Locales

local tinsert = table.insert
local tsort = table.sort
local tconcat = table.concat

local DEBUG = Aleri.SDK.Util.Debug
local INFO_PRINT = Aleri.SDK.Util.Print

local DB_LOADED = false
local PLAYER_GUID = nil
local PLAYER_NAME = nil
local PLAYER_RACE = nil
local PLAYER_CLASS = nil
local GUILD_NAME = nil
local GEARSCORE_LOADED = false
local MINIMAP_ICON_CREATED = false

local ALERI_LOADED = function()
    local l = true
    if DB_LOADED == false then
        l = false
    end
    if PLAYER_GUID == nil then
        l = false
    end
    if GUILD_NAME == nil then
        l = false
    end
    if PLAYER_NAME == nil then
        l = false
    end
    return l
end

--[==[
function Aleri.PrintPlayerXP()
	local xp = UnitXP('player')
	local xpmx = UnitXPMax('player')
	local xpr = GetXPExhaustion()
	if xpr == nil then xpr = 0 end
	return ('This Level: '..string.format("%.2f", (xp/xpmx) * 100)..'%'..' XP To Ding: '..tonumber(xpmx-xp)..' Rested XP: '..tonumber(xpr))
end

keep this for reference

function AleriInterfaceMemberDetailFrameOffSpecDD_Init()
    UIDropDownMenu_Initialize(AleriInterfaceMemberDetailFrameOffSpecDD, function(self, level, menuList)
        local info = UIDropDownMenu_CreateInfo()
        if (level or 1) == 1 then
            for class, d in pairs(Aleri.DB.Classes) do
                --info.text = tostring(d.FontStringIcon..' '..d.FontColour..L[class]..'|r') --looks too small ?
                info.text = tostring(d.FontColour..L[class]..'|r')
                --info.icon = d.Icon --overlaps the arrow
                info.hasArrow = true
                info.keepShownOnClick = false
                info.menuList = d.Specializations
                UIDropDownMenu_AddButton(info)
            end
        else
            for k, spec in pairs(menuList) do
                info.text = tostring(L[spec])
                info.isNotRadio = true
                info.keepShownOnClick = false
                info.func = function() UIDropDownMenu_SetText(AleriInterfaceMemberDetailFrameOffSpecDD, spec) end
                UIDropDownMenu_AddButton(info, level)
            end
		end
	end)
end
]==]--

Aleri.Debug = false
Aleri.InitRun = false
Aleri.InitDelay = 5 --this should allow player GUID and guild info time to be available to the client

local Config = {
    PrintFontColour = '|cff0070DE',
    ClassSummaryListView = {
        ListViewItem = {
            VerticalOffset = 30.0,
            Height = 26,
            Width = 250,
            IconSize = 28,
            IconPosX = 5,
            StatusBarPosX = 33,
            TextPosX = 150,
            TextFont = "Fonts\\FRIZQT__.TTF",
            TextInherit = "GameFontNormal",
            TextFontSize = 12,
            CountFontStringPosX = 200,
            StatusBarHeight = 22,
            StatusBarWidth = 125,
        },

    },
    GuildMemberListView = {
        ListViewItem = {
            VerticalOffset = 90,
            Height = 20,
            Width = 400,
            ClassIconSize = 20,
            ProfIconSize = 20,
            TextFont = "Fonts\\FRIZQT__.TTF",
            TextInherit = "GameFontNormal",
            TextFontSize = 11,
            ClassIconPosX = 15,
            NamePosX = 60,
            LevelPosX = 175,
            RolePosX = 220,
            InfoPosX = 265,
            MainSpecPosX = 226,
            OffSpecPosX = 334,
        },
    },
    Professions = {

    },
}

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--CHARACTER CLASS
---------------------------------------------------------------------------------------------------------------------------------------------------------------
local Character = {
    character = {
        Name = '-', 
        Level = 1, 
        ItemLevel = 0.0, 
        GearScore = 0.0, 
        Class = '-', 
        Race = '-', 
        Gender = '-',
        MainSpec = '-',
        OffSpec = '-',
        Fishing = 0, 
        Cooking = 0, 
        FirstAid = 0, 
        Profession1 = '-', 
        Profession1Level = 0, 
        Profession2 = '-', 
        Profession2Level = 0,
        INVSLOT_HEAD = 0,
        INVSLOT_NECK = 0,
        INVSLOT_SHOULDER = 0,
        INVSLOT_CHEST = 0,
        INVSLOT_WAIST = 0,
        INVSLOT_LEGS = 0,
        INVSLOT_FEET = 0,
        INVSLOT_WRIST = 0,
        INVSLOT_HAND = 0,
        INVSLOT_FINGER1 = 0,
        INVSLOT_FINGER2 = 0,
        INVSLOT_TRINKET1 = 0,
        INVSLOT_TRINKET2 = 0,
        INVSLOT_BACK = 0,
        INVSLOT_MAINHAND = 0,
        INVSLOT_OFFHAND = 0,
        INVSLOT_RANGED = 0,  
        Info = '-',
        MainCharacter = '-',
    },
    New = function(self, name, level, ilvl, gs, class, race, gender, ms, os, fish, cook, fa, prof1, prof1L, prof2, prof2L, invHead, invNeck, invShoulder, invChest, invWaist, invLeg, invFeet, invWrist, invHand, invFinger1, invFinger2, invtrinket1, invTrinket2, invBack, invMainHand, invOffHand, invRanged, info, mainChar)
        local self = {}
        self.Name = tostring(name)
        self.Level = tonumber(level) or 1
        self.ItemLevel = tonumber(ilvl)
        self.GearScore = tonumber(gs)
        self.Class = tostring(class)
        self.Race = tostring(race)
        self.Gender = tostring(gender)
        self.MainSpec = tostring(ms)
        self.OffSpec = tostring(os)
        self.Fishing = tonumber(fish)
        self.Cooking = tonumber(cook)
        self.FirstAid = tonumber(fa)
        self.Profession1 = tostring(prof1)
        self.Profession1Level = tonumber(prof1L)
        self.Profession2 = tostring(prof2)
        self.Profession2Level = tonumber(prof2L)
        self.INVSLOT_HEAD = tonumber(invHead)
        self.INVSLOT_NECK = tonumber(invNeck)
        self.INVSLOT_SHOULDER = tonumber(invShoulder)
        self.INVSLOT_CHEST = tonumber(invChest)
        self.INVSLOT_WAIST = tonumber(invWaist)
        self.INVSLOT_LEGS = tonumber(invLeg)
        self.INVSLOT_FEET = tonumber(invFeet)
        self.INVSLOT_WRIST = tonumber(invWrist)
        self.INVSLOT_HAND = tonumber(invHand)
        self.INVSLOT_FINGER1 = tonumber(invFinger1)
        self.INVSLOT_FINGER2 = tonumber(invFinger2)
        self.INVSLOT_TRINKET1 = tonumber(invtrinket1)
        self.INVSLOT_TRINKET2 = tonumber(invTrinket2)
        self.INVSLOT_BACK = tonumber(invBack)
        self.INVSLOT_MAINHAND = tonumber(invMainHand)
        self.INVSLOT_OFFHAND = tonumber(invOffHand)
        self.INVSLOT_RANGED = tonumber(invRanged)
        self.Info = tostring(info)
        self.MainCharacter = tostring(mainChar)
        --DEBUG('created new character instance for '..name)
        return setmetatable(self, character)
    end,
}
Character.character.__index = Character.character

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--view
---------------------------------------------------------------------------------------------------------------------------------------------------------------
local View = {
    SummaryFrameClassListView = {},
    RosterFrameGuildMembersListView = {},
}

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--model
---------------------------------------------------------------------------------------------------------------------------------------------------------------
local Model = {
    GuildMembers = {},
    ClassSummary = {
        { Class = 'DEATHKNIGHT', Count = 0 },
		{ Class = 'DRUID', Count = 0 },
		{ Class = 'HUNTER', Count = 0 },
		{ Class = 'MAGE', Count = 0 },
		{ Class = 'PALADIN', Count = 0 },
		{ Class = 'PRIEST', Count = 0 },
		{ Class = 'ROGUE', Count = 0 },
		{ Class = 'SHAMAN', Count = 0 },
		{ Class = 'WARLOCK', Count = 0},
		{ Class = 'WARRIOR', Count  = 0 },
    },
    ResetClassCount = function(self)
        for k, class in ipairs(self.ClassSummary) do
            class.Count = 0
        end
    end,
    UpdateClassSummary = function(self)
        self:ResetClassCount()
        local view = GetGuildRosterShowOffline() --keep the players settings
        SetGuildRosterShowOffline(true) --set to all members
        GuildRoster()
        local TotalMembers, OnlineMembers, _ = GetNumGuildMembers()
        for i = 1, TotalMembers do
            local name, rankName, rankIndex, level, classDisplayName, zone, publicNote, officerNote, isOnline, status, classID, achievementPoints, achievementRank, isMobile, canSoR, repStanding, guid = GetGuildRosterInfo(i)
            for k, class in ipairs(self.ClassSummary) do
                if class.Class == classID then
                    class.Count = class.Count + 1
                end
            end
        end
        table.sort(self.ClassSummary, function(a,b) return a.Count > b.Count end)
        for k, class in ipairs(self.ClassSummary) do
            View.SummaryFrameClassListView[class.Class]:SetPoint('TOPLEFT', 20, (((k*-1)*Config.ClassSummaryListView.ListViewItem.Height) - Config.ClassSummaryListView.ListViewItem.VerticalOffset))
            View.SummaryFrameClassListView[class.Class].StatusBar:SetValue((class.Count / TotalMembers) * 100)
            View.SummaryFrameClassListView[class.Class].Text:SetText(tostring(class.Count..' ['..string.format("%.1f",(class.Count / TotalMembers) * 100)..'%]'))
        end
        SetGuildRosterShowOffline(view) --return to player setting
    end,
    WipeGuildMembers = function(self)
        self.GuildMembers = {}
    end,
    ClearRosterFrameGuildMemberListView = function(self)
        for i = 1, 10 do
            View.RosterFrameGuildMembersListView[i]:Hide()
            View.RosterFrameGuildMembersListView[i].Character = nil
        end
    end,
    UpdateRosterFrameGuildMemberListView = function(self, key, collection)
        if (ALERI_LOADED() == true) and (type(collection) == 'table') then
            self:ClearRosterFrameGuildMemberListView()
            key = math.ceil(key)
            local i, lower, upper = 1, ((key * 10) - 9), ((key * 10) + 1)
            --for k, member in ipairs(collection) do			
            for k, member in ipairs(self.GuildMembers) do			
                if k >= lower and k < upper then
                    View.RosterFrameGuildMembersListView[i].Character = member
                    View.RosterFrameGuildMembersListView[i]:Show()
                    i = i + 1
                end
            end     
        else
            DEBUG('update roster frame listview failed - database not loaded')
        end
    end,
    ScanGuildRoster = function(self)
        if ALERI_LOADED() == true then
            DEBUG('db loaded, scanning roster')
            local showOffline = GetGuildRosterShowOffline() --keep the players settings
            SetGuildRosterShowOffline(true) --set to all members
            self:WipeGuildMembers()
            GuildRoster()
            local TotalMembers, OnlineMembers, _ = GetNumGuildMembers()
            for i = 1, TotalMembers do
                local name, rankName, rankIndex, level, classDisplayName, zone, publicNote, officerNote, isOnline, status, class, achievementPoints, achievementRank, isMobile, canSoR, repStanding, guid = GetGuildRosterInfo(i)
                class = string.upper(class)
                if class == 'DEATH KNIGHT' then class = 'DEATHKNIGHT' end
                tinsert(self.GuildMembers, { Name = name, Class = class, Level = tonumber(level), Info = publicNote, Zone = zone, Online = isOnline, AchievementPoints = tonumber(achievementPoints) } )
                if AleriGlobalSettings[GUILD_NAME].Members[name] == nil then
                    AleriGlobalSettings[GUILD_NAME].Members[name] = Character:New(name, level, 0.0, 0.0, class, '-', '-', '-', '-', 0.0, 0.0, 0.0, '-', 0.0, '-', 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, publicNote, '-')
                    DEBUG('added: '..name..' to global db during guild scan')
                end
            end
            SetGuildRosterShowOffline(showOffline) --return to player setting
            AleriInterfaceRosterFrameScrollBar:SetMinMaxValues(1, math.ceil(TotalMembers / 10))
        else
            DEBUG('scan guild roster failed - database not loaded')
        end
    end,
    GetGlobalSettingsGuildMemberCount = function(self)
        local c = 0.0
        if ALERI_LOADED() == true then
            for k, member in pairs(AleriGlobalSettings[GUILD_NAME].Members) do
                c = c + 1
            end
        end
        return tonumber(c)
    end,
    SortGuildMembers = function(self, sort)
        if ALERI_LOADED() == true then
            DEBUG('sorting roster by '..sort)
            if self.GuildMembers ~= nil then
                if (sort == 'Class') then
                    tsort(self.GuildMembers, function(a,b) 
                        if a[sort] == b[sort] then
                            if a.Level == b.Level then
                                return a.Name < b.Name
                            else
                                return (a.Level > b.Level) 
                            end
                        else 
                            return a[sort] < b[sort] 
                        end 
                    end)
                elseif (sort == 'Name') or (sort == 'Info') then
                    tsort(self.GuildMembers, function(a,b) if a[sort] == b[sort] then return (a.Level > b.Level) else return a[sort] < b[sort] end end)
                elseif (sort == 'Level') then
                    tsort(self.GuildMembers, function(a,b) if a[sort] == b[sort] then return (a.Name < b.Name) else return a[sort] > b[sort] end end)
                elseif (sort == 'Role') then
                    self:WipeGuildMembers()
                    if self:GetGlobalSettingsGuildMemberCount() > 0 then
                        DEBUG('global database members count > 0, scanning for MainSpec data')
                        for k, member in pairs(AleriGlobalSettings[GUILD_NAME].Members) do
                            if member['MainSpec'] ~= '-' then
                                DEBUG('found matching MainSpec added '..k..' to self.GuildMembers')
                                tinsert(self.GuildMembers, member)                                
                            end
                        end
                        if #self.GuildMembers > 0 then
                            DEBUG('self.GuildMember count > 0, sorting table')
                            tsort(self.GuildMembers, function(a,b) return a.MainSpec > b.MainSpec end)
                            self:UpdateRosterFrameGuildMemberListView(1, self.GuildMembers)
                            AleriInterfaceRosterFrameScrollBar:SetMinMaxValues(1, #self.GuildMembers)
                        else
                            DEBUG('self.GuildMember count < 0, no action taken')
                        end
                    end
                end
            else

            end
        end
    end,
    FilterGuildMembers = function(self, filter, sort)
        --need to implement the profile system first TODO: consider the addition of a drop down box to select certain class/spec or profs ???
        local filteredGuildMembers = {}

        UpdateRosterFrameGuildMemberListView(1, filteredGuildMembers)
    end,
    UpdateCharacterByKey = function(self, character, key, data, dataType)
        if ALERI_LOADED() == true then
            if AleriGlobalSettings[GUILD_NAME].Members[character] then
                if dataType == 'string' then
                    AleriGlobalSettings[GUILD_NAME].Members[character][key] = tostring(data)
                elseif dataType == 'number' then
                    AleriGlobalSettings[GUILD_NAME].Members[character][key] = tonumber(data)
                end
            end
        end
    end,    
    ParseCharacterData = function(self, dataString)
        DEBUG('parsing character data')
        if ALERI_LOADED() == true then
            local i = 1
            local data_sent = {}
            for d in string.gmatch(dataString, '[^:]+') do
                data_sent[i] = d
                DEBUG(i..': '..data_sent[i])
                i = i + 1
            end
            AleriGlobalSettings[GUILD_NAME].Members[data_sent[1]] = {
                Name = tostring(data_sent[1]),
                Level = tonumber(data_sent[2]),
                ItemLevel = tonumber(data_sent[3]),
                GearScore = tonumber(data_sent[4]),              
                Class = tostring(data_sent[5]),          
                Race = tostring(data_sent[6]),
                Gender = tostring(data_sent[7]),
                MainSpec = tostring(data_sent[8]),
                OffSpec = tostring(data_sent[9]),
                Fishing = tonumber(data_sent[10]),
                Cooking = tonumber(data_sent[11]),              
                FirstAid = tonumber(data_sent[12]),
                Profession1 = tostring(data_sent[13]),
                Profession1Level = tonumber(data_sent[14]),
                Profession2 = tostring(data_sent[15]),
                Profession2Level = tonumber(data_sent[16]),
                INVSLOT_HEAD = tonumber(data_sent[17]),
                INVSLOT_NECK = tonumber(data_sent[18]),
                INVSLOT_SHOULDER = tonumber(data_sent[19]),
                INVSLOT_CHEST = tonumber(data_sent[20]),
                INVSLOT_WAIST = tonumber(data_sent[21]),
                INVSLOT_LEGS = tonumber(data_sent[22]),
                INVSLOT_FEET = tonumber(data_sent[23]),
                INVSLOT_WRIST = tonumber(data_sent[24]),
                INVSLOT_HAND = tonumber(data_sent[25]),
                INVSLOT_FINGER1 = tonumber(data_sent[26]),
                INVSLOT_FINGER2 = tonumber(data_sent[27]),
                INVSLOT_TRINKET1 = tonumber(data_sent[28]),
                INVSLOT_TRINKET2 = tonumber(data_sent[29]),
                INVSLOT_BACK = tonumber(data_sent[30]),
                INVSLOT_MAINHAND = tonumber(data_sent[31]),
                INVSLOT_OFFHAND = tonumber(data_sent[32]),
                INVSLOT_RANGED = tonumber(data_sent[33]),
                Info = tostring(data_sent[34]),
                MainCharacter = tostring(data_sent[35]),
            }
        end
    end,
}
------------------------------------------------------------------------------------------------------------------------------
-- player data/functions - these collect info about the players character
local Player = {
    Character = nil,
    GetCharacterData = function(self)
        if ALERI_LOADED() == true then
            local _, class, _, race, gender, name, realm = GetPlayerInfoByGUID(UnitGUID('player'))
            if class == 'DEATH KNIGHT' then class = 'DEATHKNIGHT' end
            if gender == 3 then
                gender = 'FEMALE'
            else
                gender = 'MALE'
            end
            if self.Character == nil then
                self.Character = Character:New(name, UnitLevel('player'), 0.0, 0.0, class, race, gender, '-', '-', 0.0, 0.0, 0.0, '-', 0.0, '-', 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, '-', '-')
                INFO_PRINT('created new character sheet for: '..name, Config.PrintFontColour)
                print(name, class, race, gender)
            end
            for s = 1, GetNumSkillLines() do
                local skill, _, _, level, _, _, _, _, _, _, _, _, _ = GetSkillLineInfo(s)
                if skill == 'Fishing' then 
                    self.Character.Fishing = level
                elseif skill == 'Cooking' then
                    self.Character.Cooking = level
                elseif skill == 'First Aid' then
                    self.Character.FirstAid = level
                else
                    for k, prof in ipairs(Aleri.DB.Professions) do
                        if skill == prof.Name then
                            if self.Character.Profession1 == '-' then
                                self.Character.Profession1 = skill
                                self.Character.Profession1Level = level
                            elseif self.Character.Profession2 == '-' then
                                self.Character.Profession2 = skill
                                self.Character.Profession2Level = level
                            end
                        end
                    end
                end
            end	
            local itemlvl = 0.0
            local itemCount = 0.0
            for k, s in ipairs(Aleri.DB.InventorySlots) do
                self.Character[s.Name] = GetInventoryItemID('player', s.Id)
                if self.Character[s.Name] ~= nil then
                    local iName, iLink, iRarety, ilvl = GetItemInfo(self.Character[s.Name])
                    itemlvl = itemlvl + ilvl
                    itemCount = itemCount + 1
                else
                    self.Character[s.Name] = 0
                end
            end
            self.Character.ItemLevel = tonumber(string.format("%.2f", itemlvl/itemCount)) -- round down or format 2dp ??
            if GEARSCORE_LOADED == true then
                self.Character.GearScore = select(1, GearScore_GetScore(self.Character.Name, "player"))
            end
            Model:ScanGuildRoster()
            for k, member in ipairs(Model.GuildMembers) do
                if member.Name == self.Character.Name then
                    self.Character.Info = member.Info
                    if self.Character.Info == nil then
                        self.Character.Info = '-'
                    end
                end
            end
        end
    end,
    LoadCharacterData = function(self)
        if ALERI_LOADED() == true then
            if AleriGlobalSettings[GUILD_NAME].Members[PLAYER_NAME] == nil then
                AleriGlobalSettings[GUILD_NAME].Members[PLAYER_NAME] = self:GetCharacterData()
                INFO_PRINT(tostring('created character sheet for '..self.Character.Name), Config.PrintFontColour)
            else
                self.Character = AleriGlobalSettings[GUILD_NAME].Members[PLAYER_NAME]
                INFO_PRINT(tostring('loaded character sheet for '..self.Character.Name), Config.PrintFontColour)
                for k, v in pairs(self.Character) do
                    DEBUG(k..' '..v)
                end
            end
        end
    end,
    SaveCharacterData = function(self)
        if ALERI_LOADED() == true and self.Character.Name ~= nil then
            self:CleanMyCharacter()
            AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name] = self.Character
            INFO_PRINT('saved character data', Config.PrintFontColour)
            for k, v in pairs(AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name]) do
                DEBUG(tostring(k)..': '..tostring(v))
            end
        end
    end,
    CleanMyCharacter = function(self)
        DEBUG('cleaning character')
        for k, v in pairs(Aleri.DB.CharacterSheet) do
            if self.Character[k] == nil then
                DEBUG(tostring('missing data: '..k..' setting to default'))
                self.Character[k] = v
            end
        end
        for k, v in pairs(self.Character) do
            if v == nil or v == '' then
                DEBUG('found nil value for: '..k..' setting to 0 or -')
                if string.find(k, 'INVSLOT_') then
                    self.Character[k] = 0
                else
                    self.Character[k] = '-'
                end
            end
        end
    end,
    GenerateCharacterDataString = function(self)
        if ALERI_LOADED() == true then
            self:CleanMyCharacter()
            ds = tostring(self.Character.Name..':'..self.Character.Level..':'..self.Character.ItemLevel..':'..self.Character.GearScore..':'..self.Character.Class..':'..self.Character.Race..':'..self.Character.Gender..':'..self.Character.MainSpec..':'..self.Character.OffSpec..':'..self.Character.Fishing..':'..self.Character.Cooking..':'..self.Character.FirstAid..':'..self.Character.Profession1..':'..self.Character.Profession1Level..':'..self.Character.Profession2..':'..self.Character.Profession2Level..':'..self.Character.INVSLOT_HEAD..':'..self.Character.INVSLOT_NECK..':'..self.Character.INVSLOT_SHOULDER..':'..self.Character.INVSLOT_CHEST..':'..self.Character.INVSLOT_WAIST..':'..self.Character.INVSLOT_LEGS..':'..self.Character.INVSLOT_FEET..':'..self.Character.INVSLOT_WRIST..':'..self.Character.INVSLOT_HAND..':'..self.Character.INVSLOT_FINGER1..':'..self.Character.INVSLOT_FINGER2..':'..self.Character.INVSLOT_TRINKET1..':'..self.Character.INVSLOT_TRINKET2..':'..self.Character.INVSLOT_BACK..':'..self.Character.INVSLOT_MAINHAND..':'..self.Character.INVSLOT_OFFHAND..':'..self.Character.INVSLOT_RANGED..':'..self.Character.Info..':'..self.Character.MainCharacter)
            DEBUG(ds)
            return ds
        end
    end,
}
---------------------------------------------------------------------------------------------------------------------------------------------------------------
--minimap button
---------------------------------------------------------------------------------------------------------------------------------------------------------------
Aleri.CreateMinimapIcon = function()
    local ldb = LibStub("LibDataBroker-1.1")
    Aleri.MinimapButtonObject = ldb:NewDataObject(AleriName, {
        type = "data source",
        icon = 'Interface\\Icons\\Spell_Nature_LightningShield',
        OnClick = function(self, button)
            if button == 'LeftButton' then
                if AleriInterface:IsVisible() then
                    AleriInterface:Hide()
                else
                    AleriInterface:Show()
                end
            elseif button == "RightButton" and IsAltKeyDown() then
                StaticPopup_Show ('Aleri_Reset')
            elseif button == "RightButton" then
                -- Standard workaround call OpenToCategory twice
                -- https://www.wowinterface.com/forums/showpost.php?p=319664&postcount=2
                --InterfaceOptionsFrame_OpenToCategory(AleriName)
                --InterfaceOptionsFrame_OpenToCategory(AleriName)
            end
        end,
        OnTooltipShow = function(tooltip)
            if not tooltip or not tooltip.AddLine then return end
            tooltip:AddLine("|cff0070DE"..AleriName)
            tooltip:AddLine("|cffFFFFFFLeft click:|r Open")
            tooltip:AddLine("Alt + |cffFFFFFFRight click:|r Reload database")
        end,
    })    
    Aleri.MinimapIcon = LibStub("LibDBIcon-1.0")
    if not AleriCharacterSettings['MinimapButton'] then
        AleriCharacterSettings['MinimapButton'] = {}
    end
    Aleri.MinimapIcon:Register(AleriName, Aleri.MinimapButtonObject, AleriCharacterSettings['MinimapButton'])
end

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--slash commands
---------------------------------------------------------------------------------------------------------------------------------------------------------------
SLASH_ALERI1 = '/aleri'
SlashCmdList['ALERI'] = function(msg)
	msg = msg:lower()
	if msg == '-help' then
        INFO_PRINT(L['PrintHelp'], Aleri.PrintFontColour)  
	elseif msg == '-ui' then
        if AleriInterface:IsVisible() then
            AleriInterface:Hide()
        else
            AleriInterface:Show()
        end
    elseif msg == '-debug' then
        Aleri.Debug = not Aleri.Debug
        INFO_PRINT(tostring('debug active, '..tostring(Aleri.Debug)), Aleri.PrintFontColour)
    elseif msg == '-toggle-welcome-message' then
        if ALERI_LOADED() then
            AleriCharacterSettings['ShowWelcomeMessage'] = not AleriCharacterSettings['ShowWelcomeMessage']
            INFO_PRINT(tostring(L['WelcomeMessageToggle']..' '..tostring(AleriCharacterSettings['ShowWelcomeMessage'])))
        end
	end
end

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--xml functions
---------------------------------------------------------------------------------------------------------------------------------------------------------------
function AleriInterface_OnLoad(self)
    --had issues with this so far ???
    local version = GetAddOnMetadata(AleriName, "Version")
    INFO_PRINT(tostring('loaded - version '..version), Config.PrintFontColour)
    AleriInterfaceTab1:SetText(L['Summary'])
    AleriInterfaceTab2:SetText(L['Roster'])
    AleriInterfaceTab3:SetText(L['Professions'])
    AleriInterfaceTab4:SetText(L['Profile'])
end

function AleriInterface_OnShow(self)
    AleriInterface_Title:SetText(tostring(L['Title']..' - '..L['Summary']))
    Model:ScanGuildRoster()
    --this defaults the opening view to summary
    AleriInterfaceSummaryFrame:Show()
    AleriInterfaceRosterFrame:Hide()
    AleriInterfaceProfileFrame:Hide()
    AleriInterfaceProfessionsFrame:Hide()
    PanelTemplates_SetTab(AleriInterface, 1)
end

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--SUMMARY FRAME
function AleriInterfaceSummaryFrame_OnLoad(self)
    AleriInterfaceSummaryFrame_Header:SetText(L['SummaryHeader'])
    local k = 1
    for class, d in pairs(Aleri.DB.Classes) do
        local row = CreateFrame("FRAME", tostring('AleriInterfaceSummaryFrameRow'..k), AleriInterfaceSummaryFrame)
        row:SetSize(Config.ClassSummaryListView.ListViewItem.Width, Config.ClassSummaryListView.ListViewItem.Height)
        row:SetPoint('TOPLEFT', 20, (((k*-1)*Config.ClassSummaryListView.ListViewItem.Height) - Config.ClassSummaryListView.ListViewItem.VerticalOffset))
        row.Icon = Aleri.SDK.UI.NewIcon(row, tostring('AleriInterfaceSummaryFrame'..'_'..class..'_Icon'), 'LEFT', Config.ClassSummaryListView.ListViewItem.IconPosX, 0, Config.ClassSummaryListView.ListViewItem.IconSize, Config.ClassSummaryListView.ListViewItem.IconSize, Aleri.DB.Classes[class].Icon, true)
        row.StatusBar = Aleri.SDK.UI.NewStatusBar(row, tostring('AleriInterfaceSummaryFrame'..'_'..class..'_StatusBar'), 'LEFT', Config.ClassSummaryListView.ListViewItem.StatusBarPosX, 1, Config.ClassSummaryListView.ListViewItem.StatusBarWidth, Config.ClassSummaryListView.ListViewItem.StatusBarHeight, Aleri.DB.Classes[class].RGB, 0, 100, 50)
        row.Text = Aleri.SDK.UI.NewFontString(row, tostring('AleriInterfaceSummaryFrame'..'_'..class..'_Text'), 'LEFT', Config.ClassSummaryListView.ListViewItem.TextPosX, 0, '', nil, Config.ClassSummaryListView.ListViewItem.TextFont, Config.ClassSummaryListView.ListViewItem.TextFontSize, Config.ClassSummaryListView.ListViewItem.TextInherit)
        View.SummaryFrameClassListView[class] = row
        k = k + 1
    end
end

function AleriInterfaceSummaryFrame_OnShow(self)
    AleriInterface_Title:SetText(tostring(L['Title']..' - '..L['Summary']))
    if ALERI_LOADED() == true then
        Model:UpdateClassSummary()
    end
end

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--ROSTER FRAME
function AleriInterfaceRosterFrame_OnLoad(self)
    AleriInterfaceRosterFrameSortClass_Text:SetText(L['Class'])
    AleriInterfaceRosterFrameSortName_Text:SetText(L['Name'])
    AleriInterfaceRosterFrameSortLevel_Text:SetText(L['Level'])
    AleriInterfaceRosterFrameSortRole_Text:SetText(L['Role'])
    AleriInterfaceRosterFrameSortInfo_Text:SetText(L['Info'])
    for i = 1, 10 do
        local row = CreateFrame("FRAME", tostring('AleriInterfaceSummaryFrameRow'..i), AleriInterfaceRosterFrame)
        row:EnableMouse(true)
        row:SetSize(Config.GuildMemberListView.ListViewItem.Width, Config.GuildMemberListView.ListViewItem.Height)
        row:SetPoint('TOPLEFT', 15, (((i*-1)*Config.GuildMemberListView.ListViewItem.Height) - Config.GuildMemberListView.ListViewItem.VerticalOffset))
        row.ClassIcon = Aleri.SDK.UI.NewIcon(row, tostring('AleriInterfaceRosterFrame'..i..'_Icon'), 'LEFT', Config.GuildMemberListView.ListViewItem.ClassIconPosX, 0, Config.GuildMemberListView.ListViewItem.ClassIconSize, Config.GuildMemberListView.ListViewItem.ClassIconSize, '', true)
        row.Name =  Aleri.SDK.UI.NewFontString(row, tostring('AleriInterfaceRosterFrame'..i..'_Name'), 'LEFT', Config.GuildMemberListView.ListViewItem.NamePosX, 2, '', nil, Config.GuildMemberListView.ListViewItem.TextFont, Config.GuildMemberListView.ListViewItem.TextFontSize, Config.GuildMemberListView.ListViewItem.TextInherit)
        row.Level = Aleri.SDK.UI.NewFontString(row, tostring('AleriInterfaceRosterFrame'..i..'_Level'), 'LEFT', Config.GuildMemberListView.ListViewItem.LevelPosX, 2, '', nil, Config.GuildMemberListView.ListViewItem.TextFont, Config.GuildMemberListView.ListViewItem.TextFontSize, Config.GuildMemberListView.ListViewItem.TextInherit)
        row.Role = Aleri.SDK.UI.NewFontString(row, tostring('AleriInterfaceRosterFrame'..i..'_Level'), 'LEFT', Config.GuildMemberListView.ListViewItem.RolePosX, 2, '', nil, Config.GuildMemberListView.ListViewItem.TextFont, Config.GuildMemberListView.ListViewItem.TextFontSize, Config.GuildMemberListView.ListViewItem.TextInherit)
        row.Info = Aleri.SDK.UI.NewFontString(row, tostring('AleriInterfaceRosterFrame'..i..'_MainSpec'), 'LEFT', Config.GuildMemberListView.ListViewItem.InfoPosX, 2, '', nil, Config.GuildMemberListView.ListViewItem.TextFont, Config.GuildMemberListView.ListViewItem.TextFontSize, Config.GuildMemberListView.ListViewItem.TextInherit)
        row.Character = nil
        row:SetScript('OnShow', function(self)
            if self.Character then
                if self.Character.Class and self.Character.Name then
                    if self.Character.Class then
                        self.ClassIcon.Texture:SetTexture(Aleri.DB.Classes[self.Character.Class].Icon)
                    end
                    if self.Character.Name then
                        self.Name:SetText(tostring(Aleri.DB.Classes[self.Character.Class].FontColour..self.Character.Name))
                    end
                    if self.Character.Level then
                        self.Level:SetText(tostring(Aleri.DB.Classes[self.Character.Class].FontColour..self.Character.Level))
                    end
                    local roleString = ''
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name] then                    
                        if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainSpec ~= '-' then
                            roleString = tostring(roleString..Aleri.DB.RoleIcons[Aleri.DB.SpecToRole[self.Character.Class][AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainSpec]].FontStringIcon)
                        end
                        if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].OffSpec ~= '-' then
                            roleString = tostring(roleString..' '..Aleri.DB.RoleIcons[Aleri.DB.SpecToRole[self.Character.Class][AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].OffSpec]].FontStringIcon)
                        end
                        self.Role:SetText(roleString)
                    end
                    if self.Character.Info then
                        self.Info:SetText(tostring(self.Character.Info)) -- TODO: reconfigure the listview to show note/info better
                    end
                end
            end
        end)
        row:SetScript('OnHide', function(self)
            if self.Character then
                self.ClassIcon.Texture:SetTexture(nil)
                self.Name:SetText('')
                self.Level:SetText('')
                self.Role:SetText('')
                self.Info:SetText('')
            end
        end)
        row:SetScript('OnEnter', function(self)
            if self.Character and ALERI_LOADED() then
                GameTooltip:SetOwner(self, "ANCHOR_NONE")
                GameTooltip:SetPoint("RIGHT", self, 'LEFT')
                if self.Character.Name and self.Character.Class and self.Character.Level then
                    GameTooltip:AddDoubleLine(tostring(Aleri.DB.Classes[self.Character.Class].FontColour..self.Character.Name), tostring(Aleri.DB.Classes[self.Character.Class].FontColour..self.Character.Level..'  '..Aleri.DB.Classes[self.Character.Class].FontStringIcon))
                end
                if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name] then
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainCharacter ~= '-' then
                        GameTooltip:AddLine(' ')
                        GameTooltip:AddDoubleLine(L['MainCharacter'], AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainCharacter, 1, 1, 1, 1, 1, 1)
                    end
                    GameTooltip:AddLine(' ')
                    GameTooltip:AddDoubleLine(L['ItemLevel'], AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].ItemLevel, 1, 1, 1, 1, 1, 1)
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].GearScore then
                        GameTooltip:AddDoubleLine(L['GearScore'], AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].GearScore, 1, 1, 1, 1, 1, 1)
                    else
                        GameTooltip:AddDoubleLine(L['GearScore'], '-', 1, 1, 1, 1, 1, 1)
                    end
                    GameTooltip:AddLine(' ')
                    GameTooltip:AddLine(tostring(Aleri.DB.Classes[self.Character.Class].FontColour..L['Specializations']))
                    if  AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainSpec ~= '-' then
                        GameTooltip:AddDoubleLine(tostring(Aleri.DB.RoleIcons[Aleri.DB.SpecToRole[self.Character.Class][AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainSpec]].FontStringIcon..'|cffffffff'..AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainSpec..'|r'), Aleri.DB.SpecFontStringIcons[self.Character.Class][AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].MainSpec])
                    else
                        GameTooltip:AddLine(L['CharacterDataMissing'], 1, 1, 1)
                    end
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].OffSpec ~= '-' then
                        GameTooltip:AddDoubleLine(tostring(Aleri.DB.RoleIcons[Aleri.DB.SpecToRole[self.Character.Class][AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].OffSpec]].FontStringIcon..'|cffffffff'..AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].OffSpec..'|r '), Aleri.DB.SpecFontStringIcons[self.Character.Class][AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].OffSpec])
                    end
                    GameTooltip:AddLine(' ')
                    GameTooltip:AddLine(tostring(Aleri.DB.Classes[self.Character.Class].FontColour..L['Professions']))
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession1 ~= '-' then
                        GameTooltip:AddDoubleLine(AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession1, AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession1Level, 1, 1, 1, 1, 1, 1)
                        GameTooltip:AddTexture(Aleri.DB.ProfessionIcons[AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession1].Icon, {width = 32, height = 32})
                    end
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession2 ~= '-' then
                        GameTooltip:AddDoubleLine(AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession2, AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession2Level, 1, 1, 1, 1, 1, 1)
                        GameTooltip:AddTexture(Aleri.DB.ProfessionIcons[AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Profession2].Icon, {width = 32, height = 32})
                    end                    
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Cooking ~= '-' then
                        GameTooltip:AddDoubleLine(L['Cooking'], AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Cooking, 1, 1, 1, 1, 1, 1)
                        GameTooltip:AddTexture(Aleri.DB.ProfessionIcons.Cooking.Icon, {width = 32, height = 32})
                    end
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].FirstAid ~= '-' then
                        GameTooltip:AddDoubleLine(L['FirstAid'], AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].FirstAid, 1, 1, 1, 1, 1, 1)
                        GameTooltip:AddTexture(Aleri.DB.ProfessionIcons.FirstAid.Icon, {width = 32, height = 32})
                    end
                    if AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Fishing ~= '-' then
                        GameTooltip:AddDoubleLine(L['Fishing'],AleriGlobalSettings[GUILD_NAME].Members[self.Character.Name].Fishing, 1, 1, 1, 1, 1, 1)
                        GameTooltip:AddTexture(Aleri.DB.ProfessionIcons.Fishing.Icon, {width = 32, height = 32})
                    end
                else
                    GameTooltip:AddLine(' ')
                    GameTooltip:AddLine(L['CharacterDataMissing'], 1, 1, 1)
                end
                GameTooltip:Show()
            end
        end)
        row:SetScript('OnLeave', function(self) 
            GameTooltip_SetDefaultAnchor(GameTooltip, UIParent) 
        end)
        View.RosterFrameGuildMembersListView[i] = row
    end
end

function AleriInterfaceRosterFrame_OnShow(self)
    AleriInterface_Title:SetText(tostring(L['Title']..' - '..L['Roster']))
    AleriInterfaceRosterFrame_Header:SetText(L['RosterHeader'])
    Model:ScanGuildRoster()
    Model:SortGuildMembers('Name') -- default option?
    Model:UpdateRosterFrameGuildMemberListView(1, Model.GuildMembers)
    AleriInterfaceRosterFrameScrollBar:SetValue(1)
end

function AleriInterfaceRosterFrameSortClass_OnClick(self)
    Model:ScanGuildRoster()
    Model:SortGuildMembers('Class')
    Model:UpdateRosterFrameGuildMemberListView(1, Model.GuildMembers)
    AleriInterfaceRosterFrameScrollBar:SetValue(1) 
end

function AleriInterfaceRosterFrameSortClass_OnEnter(self)
    GameTooltip:SetOwner(self, 'ANCHOR_NONE')
    GameTooltip:SetPoint('BOTTOM', self, 'TOP')
    GameTooltip:AddLine(L['SortClass'])
    GameTooltip:Show()
end

function AleriInterfaceRosterFrameSortHeader_OnLeave(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
end

function AleriInterfaceRosterFrameSortName_OnEnter(self)
    GameTooltip:SetOwner(self, 'ANCHOR_NONE')
    GameTooltip:SetPoint('BOTTOM', self, 'TOP')
    GameTooltip:AddLine(L['SortName'])
    GameTooltip:Show()
end

function AleriInterfaceRosterFrameSortLevel_OnEnter(self)
    GameTooltip:SetOwner(self, 'ANCHOR_NONE')
    GameTooltip:SetPoint('BOTTOM', self, 'TOP')
    GameTooltip:AddLine(L['SortLevel'])
    GameTooltip:Show()
end

function AleriInterfaceRosterFrameSortRole_OnEnter(self)
    GameTooltip:SetOwner(self, 'ANCHOR_NONE')
    GameTooltip:SetPoint('BOTTOM', self, 'TOP')
    GameTooltip:AddLine(L['SortRole'])
    GameTooltip:Show()
end

function AleriInterfaceRosterFrameSortInfo_OnEnter(self)
    GameTooltip:SetOwner(self, 'ANCHOR_NONE')
    GameTooltip:SetPoint('BOTTOM', self, 'TOP')
    GameTooltip:AddLine(L['PlayerNotesInfo'])
    GameTooltip:Show()
end

function AleriInterfaceRosterFrameSortRole_OnClick(self)
    Model:ScanGuildRoster()
    Model:SortGuildMembers('Role')
    AleriInterfaceRosterFrameScrollBar:SetValue(1)
end

function AleriInterfaceRosterFrameSortLevel_OnClick(self)
    Model:ScanGuildRoster()
    Model:SortGuildMembers('Level')
    Model:UpdateRosterFrameGuildMemberListView(1, Model.GuildMembers)
    AleriInterfaceRosterFrameScrollBar:SetValue(1) 
end

function AleriInterfaceRosterFrameSortName_OnClick(self)
    Model:ScanGuildRoster()
    Model:SortGuildMembers('Name')
    Model:UpdateRosterFrameGuildMemberListView(1, Model.GuildMembers)
    AleriInterfaceRosterFrameScrollBar:SetValue(1) 
end

function AleriInterfaceRosterFrameSortInfo_OnClick(self)
    Model:ScanGuildRoster()
    Model:SortGuildMembers('Info')
    Model:UpdateRosterFrameGuildMemberListView(1, Model.GuildMembers)
    AleriInterfaceRosterFrameScrollBar:SetValue(1) 
end

function AleriInterfaceRosterFrameSortInfo_OnClick(self)

end

function AleriInterfaceRosterFrameScrollBar_OnValueChanged(self)
    if ALERI_LOADED() == true then
        Model:UpdateRosterFrameGuildMemberListView(self:GetValue(), Model.GuildMembers)
    end
end

--no need to update the view here as this invokes the ScrollBar_OnValueChanged function which will handle updating view
function AleriInterfaceRosterFrame_OnMouseWheel(self, delta)
    if ALERI_LOADED() == true then
        AleriInterfaceRosterFrameScrollBar:SetValue(AleriInterfaceRosterFrameScrollBar:GetValue() - delta) 
    end
end


---------------------------------------------------------------------------------------------------------------------------------------------------------------
--PROFILE FRAME
function AleriInterfaceProfileFrame_OnLoad(self)
    AleriInterface_Title:SetText(tostring(L['Title']..' - '..L['Profile']))
    AleriInterfaceProfileFrameHeader:SetText(L['EditCharacterInfo'])
    AleriInterfaceProfileFrameCharacterName:SetText(L['Name'])
    AleriInterfaceProfileFrameCharacterLevel:SetText(L['Level'])
    AleriInterfaceProfileFrameCharacterClass:SetText(L['Class'])
    AleriInterfaceProfileFrameCharacterMainSpec:SetText(L['MainSpec'])
    AleriInterfaceProfileFrameCharacterOffSpec:SetText(L['OffSpec'])
    AleriInterfaceProfileFrameCharacterProfession1:SetText(L['Profession1'])
    AleriInterfaceProfileFrameCharacterProfession2:SetText(L['Profession2'])
    AleriInterfaceProfileFrameCharacterFirstAid:SetText(L['FirstAid'])
    AleriInterfaceProfileFrameCharacterCooking:SetText(L['Cooking'])
    AleriInterfaceProfileFrameCharacterFishing:SetText(L['Fishing'])
    AleriInterfaceProfileFrameConfirmCharacterData:SetText(L['SaveCharacterData'])
    AleriInterfaceProfileFrameMainCharacterNameInputDesc:SetText(L['MainCharacterNameInputDesc'])
end

function AleriInterfaceProfileFrame_OnShow(self)
    if ALERI_LOADED() == true then
        Player:GetCharacterData()
        AleriInterfaceProfileFrameMainSpecDD_Init()
        UIDropDownMenu_SetWidth(AleriInterfaceProfileFrameMainSpecDD, 115)
        UIDropDownMenu_SetText(AleriInterfaceProfileFrameMainSpecDD, L['Class'])
        if Player.Character.MainSpec then
            UIDropDownMenu_SetText(AleriInterfaceProfileFrameMainSpecDD, Player.Character.MainSpec)
        end
        AleriInterfaceProfileFrameOffSpecDD_Init()
        UIDropDownMenu_SetWidth(AleriInterfaceProfileFrameOffSpecDD, 115)
        UIDropDownMenu_SetText(AleriInterfaceProfileFrameOffSpecDD, L['Class'])
        if Player.Character.OffSpec then
            UIDropDownMenu_SetText(AleriInterfaceProfileFrameOffSpecDD, Player.Character.OffSpec)
        end
        AleriInterfaceProfileFrameCharacterNameData:SetText(Player.Character.Name)
        AleriInterfaceProfileFrameCharacterLevelData:SetText(Player.Character.Level)
        AleriInterfaceProfileFrameCharacterClassData:SetText(tostring(Player.Character.Class..' '..Aleri.DB.Classes[Player.Character.Class].FontStringIcon))
        AleriInterfaceProfileFrameCharacterProfession1Data:SetText(tostring(Player.Character.Profession1..' - '..Player.Character.Profession1Level))
        AleriInterfaceProfileFrameCharacterProfession2Data:SetText(tostring(Player.Character.Profession2..' - '..Player.Character.Profession2Level))
        AleriInterfaceProfileFrameCharacterFirstAidData:SetText(Player.Character.FirstAid)
        AleriInterfaceProfileFrameCharacterCookingData:SetText(Player.Character.Cooking)
        AleriInterfaceProfileFrameCharacterFishingData:SetText(Player.Character.Fishing)
        AleriInterfaceProfileFrameMainCharacterNameInputBox:SetText(Player.Character.MainCharacter)
    end
end

function AleriInterfaceProfileFrameConfirmCharacterData_OnClick(self)
    Player:SaveCharacterData()
    SendAddonMessage('aleri-charData', Player:GenerateCharacterDataString(), 'GUILD')
end

function AleriInterfaceProfileFrameMainCharacterNameInputBox_OnTextChanged(self)
    if string.len(self:GetText()) > 0 then
        Player.Character.MainCharacter = tostring(self:GetText())
    else
        Player.Character.MainCharacter = '-'
    end
end


---------------------------------------------------------------------------------------------------------------------------------------------------------------
--PROFESSIONS FRAME
function AleriInterfaceProfessionsFrame_OnLoad(self)

end

function AleriInterfaceProfessionsFrame_OnShow(self)

end



--these 2 functions could be merged in future, pass in args(dropDown, return)
function AleriInterfaceProfileFrameMainSpecDD_Init()
    if ALERI_LOADED() == true then
        UIDropDownMenu_Initialize(AleriInterfaceProfileFrameMainSpecDD, function(self, level, menuList)
            local info = UIDropDownMenu_CreateInfo()
            for i, spec in pairs(Aleri.DB.Classes[Player.Character.Class].Specializations) do
                info.text = tostring(Aleri.DB.SpecFontStringIcons[Player.Character.Class][spec]..'  '..L[spec])
                info.hasArrow = false
                info.keepShownOnClick = false
                info.func = function() 
                    UIDropDownMenu_SetText(AleriInterfaceProfileFrameMainSpecDD, L[spec]) 
                    Player.Character.MainSpec = tostring(spec)
                    Player:SaveCharacterData()
                    SendAddonMessage('aleri-charUKV', tostring('MainSpec'..':'..spec), 'GUILD')
                end
                UIDropDownMenu_AddButton(info)
            end
        end)
    end
end
function AleriInterfaceProfileFrameOffSpecDD_Init()
    if ALERI_LOADED() == true then
        UIDropDownMenu_Initialize(AleriInterfaceProfileFrameOffSpecDD, function(self, level, menuList)
            local info = UIDropDownMenu_CreateInfo()
            for i, spec in pairs(Aleri.DB.Classes[Player.Character.Class].Specializations) do
                info.text = tostring(Aleri.DB.SpecFontStringIcons[Player.Character.Class][spec]..'  '..L[spec])
                info.hasArrow = false
                info.keepShownOnClick = false
                info.func = function()
                    UIDropDownMenu_SetText(AleriInterfaceProfileFrameOffSpecDD, L[spec]) 
                    Player.Character.OffSpec = tostring(spec)
                    Player:SaveCharacterData()
                    SendAddonMessage('aleri-charUKV', tostring('OffSpec'..':'..spec), 'GUILD')
                end
                UIDropDownMenu_AddButton(info)
            end
        end)
    end
end

function Aleri.Init()
    DEBUG('start aleri init')
    if AleriCharacterSettings == nil then
        DEBUG('character settings = nil')
        local guildName, _, _ = GetGuildInfo('player')
        if IsInGuild() and guildName then
            DEBUG('guild name'..guildName)
            local _, class, _, race, gender, name, realm = GetPlayerInfoByGUID(UnitGUID('player'))
            if class == 'DEATH KNIGHT' then class = 'DEATHKNIGHT' end
            AleriCharacterSettings = {
                GUID = UnitGUID('player'),
                MinimapButton = {},
                GuildName = guildName,
                Debug = false,
                Name = name,
                Race = race:upper(),
                Class = class:upper(),
                ShowWelcomeMessage = true,
            }
            INFO_PRINT('created character database', Config.PrintFontColour)
            PLAYER_GUID = UnitGUID('player')            
            PLAYER_NAME = name            
            GUILD_NAME = guildName
            PLAYER_RACE = AleriCharacterSettings['Race']
            PLAYER_CLASS = AleriCharacterSettings['Class']
            UIDropDownMenu_SetText(AleriInterfaceProfileFrameMainSpecDD, L['Class'])
            UIDropDownMenu_SetText(AleriInterfaceProfileFrameOffSpecDD, L['Class'])
            AleriInterfaceProfileFrameMainCharacterNameInputBox:SetText('-')
        else
            INFO_PRINT('you are not in a guild!', Config.PrintFontColour)
            DB_LOADED = false  
        end
    else
        if AleriCharacterSettings['GUID'] == nil then
            DEBUG('player guid not found in character settings')
            DB_LOADED = false
            PLAYER_GUID = nil
            PLAYER_NAME = nil
        else
            PLAYER_GUID = AleriCharacterSettings['GUID']
            PLAYER_NAME = AleriCharacterSettings['Name']
            PLAYER_RACE = AleriCharacterSettings['Race']
            PLAYER_CLASS = AleriCharacterSettings['Class']
            DEBUG('player data exists '..PLAYER_NAME)
        end
        if AleriCharacterSettings['GuildName'] == nil then
            DEBUG('player guild name not found in character settings')
            GUILD_NAME = nil
        else
            GUILD_NAME = AleriCharacterSettings['GuildName']
            DEBUG('player guild name available'..GUILD_NAME)
        end
    end
    if PLAYER_NAME and PLAYER_GUID and GUILD_NAME then
        DEBUG('character setting exists')
        DB_LOADED = true
    end   
    if AleriGlobalSettings == nil then
        AleriGlobalSettings = {}
        DEBUG('global settings = nil, created table')
    end
    if GUILD_NAME and PLAYER_NAME then
        DEBUG('guild name and player name available')
        if not AleriGlobalSettings[GUILD_NAME] then
            DEBUG('created guild table in global settings')
            AleriGlobalSettings[GUILD_NAME] = {
                Members = {
                    --[PLAYER_NAME] = Player.Character --set initial default object
                },
            }
            DB_LOADED = true
            INFO_PRINT(tostring('created database for '..GUILD_NAME), Config.PrintFontColour)
            Player:GetCharacterData()
        else
            DB_LOADED = true
            Player:LoadCharacterData()
            INFO_PRINT(tostring('loaded database for '..GUILD_NAME), Config.PrintFontColour) 
        end 
    else
        DEBUG('guild name, player name not available - database not loaded')  
        PLAYER_NAME = nil
        GUILD_NAME = nil
        DB_LOADED = false
    end
    Aleri.InitRun = true
    if ALERI_LOADED() == true and MINIMAP_ICON_CREATED == false then
        Aleri.CreateMinimapIcon()
        MINIMAP_ICON_CREATED = true
    end
    if ALERI_LOADED() == true then
        if AleriCharacterSettings['ShowWelcomeMessage'] == true then
            StaticPopup_Show('Aleri_Welcome', tostring(L['Welcome']..' '..PLAYER_NAME..' '..L['Greatest']..' '..tostring(PLAYER_CLASS:sub(1,1):upper()..PLAYER_CLASS:sub(2):lower())..' '..L['InGuild']), L['AleriWelcomeGoodPopup'])
        end
    else
        StaticPopup_Show('Aleri_Welcome', L['AleriWelcomeBadPopup'])
    end
end

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--dialogs
---------------------------------------------------------------------------------------------------------------------------------------------------------------
StaticPopupDialogs["Aleri_Reset"] = {
    text = "This will completely wipe all data for Aleri and then run the initial setup again",
    button1 = "Wipe Database",
    button2 = 'Cancel',
    OnAccept = function()
        Player.Character = nil
        Model.GuildMembers = nil
        AleriCharacterSettings = nil
        AleriGlobalSettings = nil
        Aleri.Init()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}
StaticPopupDialogs["Aleri_Welcome"] = {
    text = '%s\n\n%s',
    button1 = "Ok",
    OnAccept = function()
        DoEmote("CHEER")
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3,
}

---------------------------------------------------------------------------------------------------------------------------------------------------------------
--events
---------------------------------------------------------------------------------------------------------------------------------------------------------------
function AleriInterface_OnEvent(self, event, ...)
    if event == "ADDON_LOADED" and tostring(select(1, ...)):lower() == "aleri" then
        Aleri.SDK.Util.MakeFrameMove(AleriInterface)

    elseif event == 'GUILD_ROSTER_UPDATE' then 
        --OnShow events call a guild roster scan, is this event needed to be tracked ???

    elseif event == 'ADDON_LOADED' and tostring(select(1, ...)):lower() == 'gearscorelite' then
        GEARSCORE_LOADED = true
        INFO_PRINT(L['GearScoreDetected'])

    elseif event == 'PLAYER_GUILD_UPDATE' then
    
    elseif event == 'PLAYER_ENTERING_WORLD' then
        Aleri.LoadTime = GetTime()
        Aleri.InitRun = false

    elseif event == 'CHAT_MSG_ADDON' then
        local prefix = select(1, ...)
        local msg = select(2, ...)
        local sender = select(4, ...)        
        if prefix == 'aleri-charData' then --character dataString
            DEBUG('recieved a character data string from '..sender..' '..msg)
            Model:ParseCharacterData(msg)
        end
        if prefix == 'aleri-charUKV' then --char update key value
            local i = 1
            local data_sent = {}
            for d in string.gmatch(msg, '[^:]+') do
                data_sent[i] = d
                i = i + 1
            end
            Model:UpdateCharacterByKey(sender, data_sent[1], data_sent[2], type(data_sent[2]))
            DEBUG('character update from '..sender..' - '..data_sent[1]..': '..data_sent[2]..', dataType='..type(data_sent[2]))
        end

        
    end
end


---------------------------------------------------------------------------------------------------------------------------------------------------------------
--DIRTY HACK TO DELAY ADDON RUNNING BEFORE GUID IS AVAILABLE
---------------------------------------------------------------------------------------------------------------------------------------------------------------
function AleriInterface_OnUpdate()
    if ((Aleri.LoadTime + Aleri.InitDelay) < GetTime()) and (Aleri.InitRun == false) then
        Aleri.Init()
        AleriUpdateFrame:Hide() --should reduce update calls ???
    end
end

local f = CreateFrame('FRAME', 'AleriUpdateFrame', UIParent)
f:SetSize(10,10)
f:SetPoint('CENTER')
f:Show()
f:SetScript('OnUpdate', AleriInterface_OnUpdate)